package query

import (
	"context"
	"encoding/hex"
	"fmt"
	"time"

	"qnc-server/app/main/api/internal/svc"
	"qnc-server/app/main/api/internal/types"
	"qnc-server/app/main/model"
	"qnc-server/common/xerr"
	"qnc-server/pkg/lzkit/crypto"

	"github.com/bytedance/sonic"
	"github.com/jinzhu/copier"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
)

type QueryShareDetailLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewQueryShareDetailLogic(ctx context.Context, svcCtx *svc.ServiceContext) *QueryShareDetailLogic {
	return &QueryShareDetailLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *QueryShareDetailLogic) QueryShareDetail(req *types.QueryShareDetailReq) (resp *types.QueryShareDetailResp, err error) {
	secretKey := l.svcCtx.Config.Encrypt.SecretKey
	key, decodeErr := hex.DecodeString(secretKey)
	if decodeErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 获取AES解密解药失败, %v", err)
	}
	decryptedID, decryptErr := crypto.AesDecryptURL(req.Id, key)
	if decryptErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 解密数据失败: %v", decryptErr)
	}

	var payload types.QueryShareLinkPayload
	err = sonic.Unmarshal(decryptedID, &payload)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 解密数据失败: %v", err)
	}

	// 检查分享链接是否过期
	now := time.Now().Unix()
	if now > payload.ExpireAt {
		return &types.QueryShareDetailResp{
			Status: "expired",
		}, nil
	}

	// 获取订单信息
	order, err := l.svcCtx.OrderModel.FindOne(l.ctx, payload.OrderId)
	if err != nil {
		if errors.Is(err, model.ErrNotFound) {
			return nil, errors.Wrapf(xerr.NewErrCode(xerr.LOGIC_QUERY_NOT_FOUND), "报告查询, 订单不存在: %v", err)
		}
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.DB_ERROR), "报告查询, 查找报告错误: %v", err)
	}

	// 检查订单状态
	if order.Status != "paid" {
		return nil, errors.Wrapf(xerr.NewErrMsg("订单未支付，无法查看报告"), "")
	}

	// 获取报告信息
	queryModel, err := l.svcCtx.QueryModel.FindOneByOrderId(l.ctx, order.Id)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.DB_ERROR), "报告查询, 查找报告错误: %v", err)
	}

	var query types.Query
	query.CreateTime = queryModel.CreateTime.Format("2006-01-02 15:04:05")
	query.UpdateTime = queryModel.UpdateTime.Format("2006-01-02 15:04:05")

	processParamsErr := ProcessQueryParams(queryModel.QueryParams, &query.QueryParams, key)
	if processParamsErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告参数处理失败: %v", processParamsErr)
	}
	processErr := ProcessQueryData(queryModel.QueryData, &query.QueryData, key)
	if processErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告结果处理失败: %v", processErr)
	}
	updateFeatureAndProductFeatureErr := l.UpdateFeatureAndProductFeature(queryModel.ProductId, &query.QueryData)
	if updateFeatureAndProductFeatureErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告结果处理失败: %v", updateFeatureAndProductFeatureErr)
	}
	// 复制报告数据
	err = copier.Copy(&query, queryModel)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告结构体复制失败, %v", err)
	}
	product, err := l.svcCtx.ProductModel.FindOne(l.ctx, queryModel.ProductId)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 获取商品信息失败, %v", err)
	}
	query.ProductName = product.ProductName
	return &types.QueryShareDetailResp{
		Status: "success",
		Query:  query,
	}, nil
}

func (l *QueryShareDetailLogic) UpdateFeatureAndProductFeature(productID int64, target *[]types.QueryItem) error {
	// 遍历 target 数组，使用倒序遍历，以便删除元素时不影响索引
	for i := len(*target) - 1; i >= 0; i-- {
		queryItem := &(*target)[i]

		// 确保 Data 为 map 类型
		data, ok := queryItem.Data.(map[string]interface{})
		if !ok {
			return fmt.Errorf("queryItem.Data 必须是 map[string]interface{} 类型")
		}

		// 从 Data 中获取 apiID
		apiID, ok := data["apiID"].(string)
		if !ok {
			return fmt.Errorf("queryItem.Data 中的 apiID 必须是字符串类型")
		}

		// 查询 Feature
		feature, err := l.svcCtx.FeatureModel.FindOneByApiId(l.ctx, apiID)
		if err != nil {
			// 如果 Feature 查不到，也要删除当前 QueryItem
			*target = append((*target)[:i], (*target)[i+1:]...)
			continue
		}

		// 查询 ProductFeatureModel
		builder := l.svcCtx.ProductFeatureModel.SelectBuilder().Where("product_id = ?", productID)
		productFeatures, err := l.svcCtx.ProductFeatureModel.FindAll(l.ctx, builder, "")
		if err != nil {
			return fmt.Errorf("查询 ProductFeatureModel 错误: %v", err)
		}

		// 遍历 productFeatures，找到与 feature.ID 关联且 enable == 1 的项
		var featureData map[string]interface{}
		// foundFeature := false
		sort := 0
		for _, pf := range productFeatures {
			if pf.FeatureId == feature.Id { // 确保和 Feature 关联
				sort = int(pf.Sort)
				break // 找到第一个符合条件的就退出循环
			}
		}
		featureData = map[string]interface{}{
			"featureName": feature.Name,
			"sort":        sort,
		}

		// 更新 queryItem 的 Feature 字段（不是数组）
		queryItem.Feature = featureData
	}

	return nil
}
